import random
import string
import time
import hashlib
import re
from datetime import datetime, timedelta
import logging
from typing import Optional, Tuple
from .database import get_cursor, get_connection
import psycopg2
from psycopg2.extras import RealDictCursor

logger = logging.getLogger(__name__)

class RedXCode:
    def __init__(self, db_url=None):
        self.entropy_pool = string.ascii_uppercase + string.digits
        self.max_attempts = 5
        self.db_url = db_url
        self._conn = None
        self.lifetime_date = datetime(9999, 12, 31)  # Far future date for lifetime codes

    def _validate_email(self, email: str) -> bool:
        """Validate email format"""
        pattern = r'^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$'
        return bool(re.match(pattern, email))

    def Generate(self, email: str, length: int = 8, expiry_minutes: int = 15, 
                base_time: Optional[datetime] = None) -> Optional[Tuple[str, str, str]]:
        """Generate a secure random code"""
        try:
            # Validate inputs
            if not email or not self._validate_email(email):
                logger.error(f"Invalid email format: {email}")
                return None

            if not isinstance(length, int) or not isinstance(expiry_minutes, int):
                logger.error("Invalid length or expiry_minutes type")
                return None

            self._cleanup_expired_codes()

            # Enforce length limits
            length = max(8, min(20, length))
            expiry_minutes = max(1, min(60, expiry_minutes))

            creation_time = base_time if base_time else datetime.now()
            expiration_time = self.lifetime_date if expiry_minutes == 0 else creation_time + timedelta(minutes=expiry_minutes)

            attempts = 0
            while attempts < self.max_attempts:
                # Generate secure random code
                seed = int(time.time() * 1000) + random.randint(1, 1000000)
                random.seed(seed)

                code = ''.join(random.choices(self.entropy_pool, k=length - 2))
                checksum = hashlib.sha256(code.encode()).hexdigest()[:2].upper()
                full_code = code + checksum

                try:
                    with self._get_cursor() as cursor:
                        # Check for existing valid codes for this email
                        cursor.execute("""
                            SELECT COUNT(*) as count FROM verification_codes 
                            WHERE email = %s AND NOT used AND expires_at > NOW()
                        """, (email,))
                        result = cursor.fetchone()
                        if result and result['count'] > 0:
                            logger.warning(f"Active code already exists for {email}")
                            return None

                        # Store the new code
                        cursor.execute("""
                            INSERT INTO verification_codes 
                            (email, code, created_at, expires_at)
                            VALUES (%s, %s, %s, %s)
                            RETURNING id
                        """, (email, full_code, creation_time, expiration_time))
                        cursor.connection.commit()

                        if cursor.fetchone():
                            logger.info(f"Generated verification code for {email}")
                            return (
                                full_code,
                                creation_time.strftime("%m/%d/%Y/%I:%M%p").lower(),
                                "never" if expiry_minutes == 0 else expiration_time.strftime("%m/%d/%Y/%I:%M%p").lower()
                            )
                except psycopg2.Error as e:
                    logger.error(f"Database error: {e}")
                    if "duplicate key value violates unique constraint" in str(e):
                        attempts += 1
                        continue
                    raise

                attempts += 1

            logger.error("Failed to generate unique code after maximum attempts")
            return None

        except Exception as e:
            logger.error(f"Error generating verification code: {e}")
            return None

    def _get_connection(self):
        """Get database connection using either internal or external configuration"""
        if not self._conn or self._conn.closed:
            if self.db_url:
                self._conn = psycopg2.connect(self.db_url)
            else:
                self._conn = get_connection()
        return self._conn

    def _get_cursor(self):
        """Get database cursor with proper configuration"""
        return self._get_connection().cursor(cursor_factory=RealDictCursor)

    def _cleanup_expired_codes(self):
        """Remove expired verification codes from the database"""
        try:
            with self._get_cursor() as cursor:
                cursor.execute("""
                    DELETE FROM verification_codes 
                    WHERE expires_at < NOW()
                """)
                cursor.connection.commit()
                cleaned = cursor.rowcount > 0
                if cleaned:
                    logger.info(f"Cleaned up {cursor.rowcount} expired verification codes")
        except Exception as e:
            logger.error(f"Error cleaning up expired codes: {e}")

    def IsValid(self, code_or_result: str | Tuple[str, str, str]) -> bool:
        """Check if a verification code is still valid"""
        self._cleanup_expired_codes()

        # Extract the code from input
        if isinstance(code_or_result, tuple):
            code = code_or_result[0]
        else:
            code = code_or_result

        try:
            with self._get_cursor() as cursor:
                cursor.execute("""
                    SELECT EXISTS(
                        SELECT 1 FROM verification_codes
                        WHERE code = %s 
                        AND NOT used 
                        AND expires_at > NOW()
                        AND attempts < %s
                    )
                """, (code, self.max_attempts))
                result = cursor.fetchone()
                return result['exists'] if result else False
        except Exception as e:
            logger.error(f"Error checking code status: {e}")
            return False

    def Initiate(self, code_or_result: str | Tuple[str, str, str]) -> bool:
        """Activate a verification code"""
        self._cleanup_expired_codes()

        # Extract the code from input
        if isinstance(code_or_result, tuple):
            code = code_or_result[0]
        else:
            code = code_or_result

        try:
            with self._get_cursor() as cursor:
                cursor.execute("""
                    UPDATE verification_codes
                    SET used = TRUE
                    WHERE code = %s 
                    AND expires_at > NOW()
                    AND NOT used
                    RETURNING id
                """, (code,))
                cursor.connection.commit()
                return cursor.fetchone() is not None
        except Exception as e:
            logger.error(f"Error initiating code: {e}")
            return False

    def RemoveData(self, email: str) -> bool:
        """Remove all verification codes for a given email address"""
        try:
            with self._get_cursor() as cursor:
                cursor.execute("""
                    DELETE FROM verification_codes 
                    WHERE email = %s
                """, (email,))
                cursor.connection.commit()
                deleted = cursor.rowcount > 0
                if deleted:
                    logger.info(f"Removed all verification codes for {email}")
                return deleted
        except Exception as e:
            logger.error(f"Error removing verification codes: {e}")
            return False

    def __del__(self):
        """Cleanup database connection on object destruction"""
        if self._conn and not self._conn.closed:
            self._conn.close()