import psycopg2
from psycopg2.extras import RealDictCursor
from contextlib import contextmanager
from .config import CONFIG
import logging

logger = logging.getLogger(__name__)

def get_connection():
    """Get a database connection with proper error handling"""
    try:
        return psycopg2.connect(
            host=CONFIG['db'].host,
            port=CONFIG['db'].port,
            database=CONFIG['db'].database,
            user=CONFIG['db'].user,
            password=CONFIG['db'].password
        )
    except Exception as e:
        logger.error(f"Database connection error: {e}")
        raise

@contextmanager
def get_cursor():
    """Context manager for database cursor with automatic commit/rollback"""
    conn = get_connection()
    try:
        yield conn.cursor(cursor_factory=RealDictCursor)
        conn.commit()
    except Exception as e:
        conn.rollback()
        logger.error(f"Database operation error: {e}")
        raise
    finally:
        conn.close()

def init_db():
    """Initialize database tables"""
    with get_cursor() as cursor:
        # Create verification_codes table
        cursor.execute("""
            CREATE TABLE IF NOT EXISTS verification_codes (
                id SERIAL PRIMARY KEY,
                email VARCHAR(255) NOT NULL,
                code VARCHAR(255) NOT NULL,
                created_at TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                expires_at TIMESTAMP NOT NULL,
                used BOOLEAN DEFAULT FALSE,
                attempts INTEGER DEFAULT 0,
                UNIQUE(code, used)
            );

            CREATE INDEX IF NOT EXISTS idx_verification_codes_email 
                ON verification_codes(email);

            CREATE INDEX IF NOT EXISTS idx_verification_codes_active 
                ON verification_codes(code) 
                WHERE NOT used;
        """)

        # Create rate_limits table
        cursor.execute("""
            CREATE TABLE IF NOT EXISTS rate_limits (
                id SERIAL PRIMARY KEY,
                email VARCHAR(255) NOT NULL,
                endpoint VARCHAR(255) NOT NULL,
                request_count INTEGER DEFAULT 1,
                window_start TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
                UNIQUE(email, endpoint)
            );

            CREATE INDEX IF NOT EXISTS idx_rate_limits_window 
                ON rate_limits(email, endpoint, window_start);
        """)

        # Recreate redx_subscriptions table with simple schema
        cursor.execute("""
            DROP TABLE IF EXISTS redx_subscriptions CASCADE;

            CREATE TABLE redx_subscriptions (
                id SERIAL PRIMARY KEY,
                email VARCHAR(255) NOT NULL,
                start_time VARCHAR(255) NOT NULL,
                expiration_time VARCHAR(255) NOT NULL,
                subscription_name VARCHAR(255) NOT NULL,
                subscription_type VARCHAR(255) NOT NULL,
                active BOOLEAN DEFAULT TRUE,
                UNIQUE(email)
            );

            CREATE INDEX IF NOT EXISTS idx_redx_subscriptions_email 
                ON redx_subscriptions(email);

            CREATE INDEX IF NOT EXISTS idx_redx_subscriptions_expiration 
                ON redx_subscriptions(expiration_time);
        """)

        logger.info("Database tables initialized successfully")