import smtplib
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from .config import CONFIG
import logging

logger = logging.getLogger(__name__)

class EmailService:
    def __init__(self):
        self.smtp_server = CONFIG['email']['smtp_server']
        self.smtp_port = CONFIG['email']['smtp_port']
        self.sender_email = CONFIG['email']['sender_email']
        self.app_password = CONFIG['email']['password']

        if not all([self.sender_email, self.app_password]):
            logger.error("Email configuration is incomplete. Missing required credentials.")
            raise ValueError("Email configuration is incomplete. Please set SENDER_EMAIL and EMAIL_APP_PASSWORD")

    def send_verification_code(self, recipient_email: str, code: str) -> bool:
        """Send a verification code email"""
        try:
            if not all([self.smtp_server, self.smtp_port, self.sender_email, self.app_password]):
                logger.error("Email configuration is incomplete")
                return False

            subject = "Your RedXAuth Verification Code"
            body = f"""
            Your verification code is: {code}

            This code will expire in 15 minutes.
            Do not share this code with anyone.

            Powered by RedXVerification.API
            """

            success = self._send_email(recipient_email, subject, body)
            if success:
                logger.info(f"Verification code sent successfully to {recipient_email}")
            else:
                logger.error(f"Failed to send verification code to {recipient_email}")
            return success

        except Exception as e:
            logger.error(f"Failed to send verification code: {e}")
            return False

    def RedXSubEmail(self, recipient_email: str, expiration_time: str, custom_message: str = "") -> bool:
        """Send a subscription confirmation email"""
        try:
            subject = "Your RedXAuth Subscription"
            body = f"""
            Thank you for subscribing to RedXAuth!

            Your subscription is now active and will expire on: {expiration_time}

            {custom_message if custom_message else ""}

            Benefits include:
            - Unlimited verifications
            - Priority support
            - Access to premium features

            Powered by RedXVerification.API
            """
            success = self._send_email(recipient_email, subject, body)
            if success:
                logger.info(f"Subscription email sent successfully to {recipient_email}")
            return success
        except Exception as e:
            logger.error(f"Failed to send subscription email: {e}")
            return False

    def CodeAuth(self, recipient_email: str, expiry_time: str, code: str, message: str = "") -> bool:
        """Alias for send_verification_code to maintain compatibility"""
        try:
            success = self.send_verification_code(recipient_email, code)
            if success:
                logger.info(f"Auth code sent successfully via CodeAuth to {recipient_email}")
            return success
        except Exception as e:
            logger.error(f"Failed to send auth code via CodeAuth: {e}")
            return False

    def _send_email(self, recipient_email: str, subject: str, message: str) -> bool:
        """Internal method to send emails"""
        msg = MIMEMultipart()
        msg['From'] = self.sender_email
        msg['To'] = recipient_email
        msg['Subject'] = subject
        msg.attach(MIMEText(message.strip(), 'plain'))

        try:
            with smtplib.SMTP(self.smtp_server, self.smtp_port) as server:
                server.starttls()
                server.login(self.sender_email, self.app_password)
                server.send_message(msg)
                logger.info(f"Email sent successfully to {recipient_email}")
                return True
        except Exception as e:
            logger.error(f"Failed to send email: {e}")
            return False

# Keep the RedXVerifyEmail class for backward compatibility
RedXVerifyEmail = EmailService