from datetime import datetime, timedelta
from typing import Optional, Dict
from .database import get_cursor
from .email_service import EmailService
from .config import CONFIG

class SubscriptionService:
    def __init__(self):
        self.email_service = EmailService()

    def format_datetime(self, dt: datetime) -> str:
        """Format datetime to MM/DD/YYYY/HH:MMam/pm"""
        return dt.strftime("%m/%d/%Y/%I:%M%p").lower()

    def create_subscription(self, email: str, plan_type: str, duration_days: int, custom_message: str = "") -> Dict:
        """Create a new subscription, ensuring only one active subscription per user"""
        start_date = datetime.now()
        end_date = start_date + timedelta(days=duration_days)

        # Format dates in the required format
        start_time = self.format_datetime(start_date)
        expiration_time = self.format_datetime(end_date)

        with get_cursor() as cursor:
            # Check if user already has an active subscription
            cursor.execute("""
                SELECT id FROM redx_subscriptions
                WHERE email = %s AND active = TRUE
            """, (email,))

            existing = cursor.fetchone()
            if existing:
                return {"error": "User already has an active subscription"}

            # Create new subscription
            cursor.execute("""
                INSERT INTO redx_subscriptions
                (email, start_time, expiration_time, subscription_name, subscription_type)
                VALUES (%s, %s, %s, %s, %s)
                RETURNING id, email, start_time, expiration_time, subscription_name, subscription_type, active
            """, (email, start_time, expiration_time, f"{plan_type} Plan", plan_type))

            result = cursor.fetchone()
            if not result:
                return {"error": "Failed to create subscription"}

            subscription_data = dict(result)

        # Send subscription email with custom message
        self.email_service.RedXSubEmail(
            email,
            expiration_time,
            custom_message
        )

        return subscription_data

    def check_subscription(self, email: str) -> Dict:
        """Check if user has an active subscription"""
        with get_cursor() as cursor:
            cursor.execute("""
                SELECT * FROM redx_subscriptions
                WHERE email = %s AND active = TRUE
                ORDER BY expiration_time DESC
                LIMIT 1
            """, (email,))

            result = cursor.fetchone()
            if result:
                return dict(result)
            return {"active": False}

    def resign_subscription(self, email: str) -> bool:
        """Cancel active subscription for an email address."""
        with get_cursor() as cursor:
            cursor.execute("""
                UPDATE redx_subscriptions
                SET active = FALSE
                WHERE email = %s AND active = TRUE
            """, (email,))

            return cursor.rowcount > 0