"""
Command Line Interface for RedXAuth Tutorial
Provides an interactive way to run through the tutorial
"""

import sys
import time
from typing import Any, Dict
import logging

from .interactive_guide import TutorialGuide

def print_step(step_data: Dict[str, Any]) -> None:
    """Print tutorial step information"""
    print("\n" + "="*50)
    print(f"Step {step_data['step']} of {step_data.get('total_steps', 4)}")
    print("="*50)
    print("\n" + step_data['title'])
    print("-"*len(step_data['title']))
    print(step_data['description'])
    
    if step_data.get('example'):
        print("\nExample:")
        print(step_data['example'])
    
    if step_data.get('action_required'):
        print(f"\nAction Required: {step_data['action_required']}")

def handle_error(error: str) -> None:
    """Handle and display errors"""
    print("\n❌ Error:", error)
    print("Please try again or type 'exit' to quit")

def main() -> None:
    """Main CLI entry point"""
    print("""
🔐 Welcome to RedXAuth Tutorial! 🔐
This interactive guide will help you learn the basics of RedXAuth.
Type 'exit' at any time to quit.
    """)
    
    tutorial = TutorialGuide()
    current_data = tutorial.start()
    
    while True:
        print_step(current_data)
        
        if not current_data.get('action_required'):
            print("\nTutorial completed! Thank you for learning RedXAuth!")
            break
            
        action = current_data['action_required']
        user_input = input("\nEnter your response (or 'exit' to quit): ").strip()
        
        if user_input.lower() == 'exit':
            print("\nExiting tutorial. Thanks for trying RedXAuth!")
            break
            
        try:
            if action == 'set_test_email':
                current_data = tutorial.set_test_email(user_input)
            elif action == 'generate_code':
                current_data = tutorial.generate_code()
            elif action == 'send_verification':
                if 'code' in current_data:
                    current_data = tutorial.send_verification(current_data['code'])
                else:
                    handle_error("No verification code available")
                    continue
            elif action == 'create_subscription':
                current_data = tutorial.create_subscription()
            else:
                handle_error(f"Unknown action: {action}")
                continue
                
            if 'error' in current_data:
                handle_error(current_data['error'])
                continue
                
        except Exception as e:
            handle_error(str(e))
            continue

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    try:
        main()
    except KeyboardInterrupt:
        print("\nTutorial interrupted. Thanks for trying RedXAuth!")
        sys.exit(0)
