"""
Interactive Tutorial Guide for RedXAuthCodes
Helps users learn and implement authentication features step by step
"""

import os
from typing import Optional, Dict, Any
from datetime import datetime, timedelta
import logging

from ..core.code_generator import RedXCode
from ..core.email_service import EmailService
from ..core.subscription_service import SubscriptionService

logger = logging.getLogger(__name__)

class TutorialGuide:
    def __init__(self):
        """Initialize the tutorial guide with required services"""
        self.code_generator = RedXCode()
        self.email_service = EmailService()
        self.subscription_service = SubscriptionService()
        self.current_step = 0
        self.total_steps = 4
        self.test_email: Optional[str] = None
        
    def start(self) -> Dict[str, Any]:
        """Start the interactive tutorial"""
        self.current_step = 1
        return {
            'step': self.current_step,
            'total_steps': self.total_steps,
            'title': 'Welcome to RedXAuth Tutorial!',
            'description': '''
Let's get started with RedXAuth! This tutorial will guide you through:
1. Generating verification codes
2. Sending email verifications
3. Managing subscriptions
4. Integrating payments

To begin, please provide a test email address.
''',
            'action_required': 'set_test_email',
            'example': 'test@example.com'
        }

    def set_test_email(self, email: str) -> Dict[str, Any]:
        """Set the test email for the tutorial"""
        self.test_email = email
        self.current_step = 2
        
        # Clean up any existing test data
        try:
            self.code_generator.RemoveData(email)
            logger.info(f"Cleaned up existing data for {email}")
        except Exception as e:
            logger.warning(f"Cleanup warning: {e}")

        return {
            'step': self.current_step,
            'title': 'Step 1: Generate Verification Code',
            'description': '''
Great! Now let's generate your first verification code.
This code will be:
- 8 characters long
- Valid for 15 minutes
- Sent to your email

Ready to generate the code?
''',
            'action_required': 'generate_code',
            'example': '''
# Example code:
code_result = code_generator.Generate(
    email="test@example.com",
    length=8,
    expiry_minutes=15
)
'''
        }

    def generate_code(self) -> Dict[str, Any]:
        """Generate a verification code"""
        if not self.test_email:
            return {'error': 'Please set a test email first'}
        
        try:
            code_result = self.code_generator.Generate(
                email=self.test_email,
                length=8,
                expiry_minutes=15
            )
            
            if not code_result:
                return {'error': 'Failed to generate code'}
                
            code, created_at, expires_at = code_result
            self.current_step = 3
            
            return {
                'step': self.current_step,
                'title': 'Step 2: Email Verification',
                'description': f'''
Success! Generated verification code: {code}
Created at: {created_at}
Expires at: {expires_at}

Now, let's send this code via email.
The EmailService will:
- Format the email with your code
- Send it securely
- Track delivery status

Ready to send the verification email?
''',
                'action_required': 'send_verification',
                'code': code,
                'example': '''
# Example code:
email_service.send_verification_code(
    recipient_email="test@example.com",
    code="YOUR_CODE"
)
'''
            }
        except Exception as e:
            logger.error(f"Code generation error: {e}")
            return {'error': f'Failed to generate code: {str(e)}'}

    def send_verification(self, code: str) -> Dict[str, Any]:
        """Send verification email"""
        if not self.test_email:
            return {'error': 'Please set a test email first'}
            
        try:
            email_sent = self.email_service.send_verification_code(
                self.test_email,
                code
            )
            
            if not email_sent:
                return {'error': 'Failed to send verification email'}
                
            self.current_step = 4
            
            return {
                'step': self.current_step,
                'title': 'Step 3: Subscription Management',
                'description': f'''
Great! Verification email sent to {self.test_email}

Now let's create a subscription. You can:
- Create different subscription plans
- Set custom durations
- Add welcome messages
- Track subscription status

Ready to create a test subscription?
''',
                'action_required': 'create_subscription',
                'example': '''
# Example code:
subscription = subscription_service.create_subscription(
    email="test@example.com",
    plan_type="Premium",
    duration_days=30,
    custom_message="Welcome to Premium!"
)
'''
            }
        except Exception as e:
            logger.error(f"Email sending error: {e}")
            return {'error': f'Failed to send email: {str(e)}'}

    def create_subscription(self) -> Dict[str, Any]:
        """Create a test subscription"""
        if not self.test_email:
            return {'error': 'Please set a test email first'}
            
        try:
            subscription = self.subscription_service.create_subscription(
                email=self.test_email,
                plan_type="Tutorial",
                duration_days=7,  # 7-day trial
                custom_message="Welcome to the RedXAuth Tutorial!"
            )
            
            if not subscription:
                return {'error': 'Failed to create subscription'}
                
            self.current_step = 5  # Tutorial complete
            
            return {
                'step': self.current_step,
                'title': 'Tutorial Complete!',
                'description': f'''
Congratulations! You've completed the RedXAuth tutorial!

You've learned how to:
✓ Generate secure verification codes
✓ Send verification emails
✓ Manage subscriptions

Your test subscription details:
Email: {self.test_email}
Plan: Tutorial
Duration: 7 days
Status: Active

Next Steps:
1. Check out the full documentation
2. Integrate with your application
3. Customize email templates
4. Set up payment processing

Need help? Visit our documentation at:
https://github.com/redxdev/redxauthcodes
''',
                'action_required': None,
                'subscription': subscription
            }
            
        except Exception as e:
            logger.error(f"Subscription creation error: {e}")
            return {'error': f'Failed to create subscription: {str(e)}'}

    def get_progress(self) -> Dict[str, Any]:
        """Get current tutorial progress"""
        return {
            'current_step': self.current_step,
            'total_steps': self.total_steps,
            'progress_percentage': (self.current_step / self.total_steps) * 100
        }
